#ifndef H_MSGS
#define H_MSGS
/*
 * msgs.h
 *
 * Enumeration for text in BINKLEY.LNG
 *
 *  27Jan91: SWG : Converted to enumerations to make updating it easier
 *
 * IMPORTANT: This MUST match up exactly with BINKLEY.LNG as produced by BTLANG
 */

typedef enum {
	M_DIALING_SCRIPT,
	M_NOOPEN_SCRIPT,
	M_MASTER_SCRIPT_TIMER,
	M_SCRIPT_FAILED,
	M_NO_BREAK,
	M_TOO_MANY_LABELS,
	M_MACRO_NUMBER,
	M_OUT_OF_RANGE,
	M_SHELL_NUMBER,
	M_ILLEGAL_ZONE,
	M_REQUESTS_ALLOWED,
	M_BAD_LOGLEVEL,
	M_BAD_MAXPORT,
	M_ILLEGAL_PORT,
	M_ILLEGAL_CARRIER,
	M_KEY,
	M_CALL,
	M_UNKNOWN_LINE,
	M_BAD_DIR,
	M_FAIL_EXEC,
#ifdef NEW
    M_CREATED_DIR,
    M_NO_CREATE_DIR,
#endif
	M_PRESS_ESCAPE,
	M_NO_BBS,
	M_NOTHING_TO_SEND,
	M_CONNECT_ABORTED,
	M_MODEM_HANGUP,
	M_NO_OUT_REQUESTS,
	M_OUT_REQUESTS,
	M_END_OUT_REQUESTS,
	M_FREQ_DECLINED,
	M_ADDRESS,
	M_INTRO,
	M_NOBODY_HOME,
	M_NO_CARRIER,
	M_PROTECTED_SESSION,
	M_PWD_ERROR,
	M_CALLED,
	M_WAZOO_METHOD,
	M_WAZOO_END,
	M_PACKET_MSG,
	M_OPEN_MSG,
	M_KBD_MSG,
	M_TRUNC_MSG,
	M_RENAME_MSG,
	M_DEVICE_MSG,
	M_FUBAR_MSG,
	M_UNLINKING_MSG,
	M_CAN_MSG,
	M_NO_CTL_FILE,
	M_FOSSIL_GONE,
	M_NO_LOGFILE,
	M_FOSSIL_TYPE,
	M_FOSSIL_REV3,
	M_STARTING_EVENT,
	M_EVENT_EXIT,
	M_CLEAN_PACK,
	M_AFTER_CLEAN_PACK,
	M_DATE_PROBLEM,
	M_BBS_EXIT,
	M_BBS_SPAWN,
	M_BBS_RETURN,
	M_EXT_MAIL,
	M_SETTING_BAUD,
	M_REMOTE_USES,
	M_VERSION,
	M_PROGRAM,
	M_SEND_FALLBACK,
	M_REFUSING_IN_FREQ,
	M_TOO_LONG,
	M_0001_END,
	M_RECV_FALLBACK,
	M_GIVING_MAIL,
	M_REFUSE_PICKUP,
	M_MEM_ERROR,
	M_OUTBOUND,
	M_FILE_ATTACHES,
	M_MAKING_FREQ,
	M_END_OF,
	M_RECV_MAIL,
	M_NO_PICKUP,
	M_INBOUND,
	M_MAIL_PACKET,
	M_PWD_ERR_ASSUMED,
	M_CANT_RENAME_MAIL,
	M_MAIL_PACKET_RENAMED,
	M_NO_PROTOCOL,
	M_DUP_PROTOCOL,
	M_PROCESSING_NODE,
	M_NO_ADDRESS,
	M_NUISANCE_CALLER,
	M_NO_BOSS,
	M_UNABLE_TO_OPEN,
	M_NODELIST_MEM,
	M_NODELIST_READ_ERR,
	M_NODELIST_SEEK_ERR,
	M_NODELIST_REC_ERR,
	M_FILE_REQUESTS,
	M_MATCHING_FILES,
	M_BAD_BARK,
	M_BAD_CRC,
	M_MISCONFIGURED,
	M_EVENT_WAITING,
	M_INCOMING_CALL,
	M_DYNAMIC_EVENT,
	M_EXIT_REQUEST,
	M_FUNCTION_KEY,
	M_DISABLE_MODEM,
	M_BEGIN_MESSAGE_READER,
	M_END_MESSAGE_READER,
	M_ENABLE_MODEM,
	M_NO_MESSAGE_READER,
	M_NO_GET,
	M_NO_KILL,
	M_POLL_MODE,
	M_POLL_COMPLETED,
	M_NO_SEND,
	M_ENTER_TERMINAL_MODE,
	M_POLLING_BOSS,
	M_KEYBOARD_SHELL,
	M_END_KEYBOARD_SHELL,
	M_NO_KEYBOARD_SHELL,
	M_JUNK_CHARACTER,
	M_SHELLING,
	M_TYPE_EXIT,
	M_BINKLEY_BACK,
	M_NO_CALLS_NOW,
	M_IMMEDIATE_CALL,
	M_ENTER_NET_NODE,
	M_CURRENTLY_POLLING,
	M_READY_INIT,
	M_READY_WAITING,
	M_READY_DIALING,
	M_READY_HANGUP,
	M_READY_CONNECT,
	M_DIALING_NUMBER,
	M_NO_END_MIDNIGHT,
	M_INVALID_START,
	M_INVALID_END,
	M_INVALID_AVGWAIT,
	M_BAD_ERRORLEVEL,
	M_BAD_TRIES,
	M_INDECIPHERABLE,
	M_BAD_DAY,
	M_NOTHING_IN_OUTBOUND,
	M_OUTBOUND_HEADER,
	M_INSUFFICIENT_DATA,
	M_END_OF_ATTEMPT,
	M_STILL_HAVE_MAIL,
	M_EXIT_COMPRESSED,
	M_EXIT_AFTER_EXTENT,
	M_EXIT_AFTER_MAIL,
	M_AFTERMAIL,
	M_OK_AFTERMAIL,
	M_SETTING,
	M_INITIAL_SPEED,
	M_INTRO_END,
	M_MULTI_TASKER,
	M_CURRENT_SETTINGS,
	M_TODAY_GLANCE,
	M_PENDING_OUTBOUND,
	M_INITIALIZING_SYSTEM,
	M_RECENT_ACTIVITY,
	M_TRANSFER_STATUS,
	M_EVENT_COLON,
	M_PORT_COLON,
	M_STATUS_INIT,
	M_BBS_MAIL,
	M_CALLS_OUT,
	M_GOOD_COST,
	M_FILES_IO,
	M_LAST,
	M_ALTF10,
	M_NODE_COLON,
	M_POINT_COLON,
	M_PASSWORD_OVERRIDE,
	M_ZMODEM_STARTED,
	M_ZMODEM_FINISHED,
	M_BINK_NOW_AT,
	M_PHONE_OR_NODE,
	M_NO_DROP_DTR,
	M_WELCOME_BACK,
	M_INPUT_LOGFILE,
	M_MAILER_MODE,
	M_MANUAL_MODE,
	M_BINK_NOW_USING,
	M_READY_TO_SEND,
	M_WHICH_PROTOCOL,
	M_READY_TO_RECEIVE,
	M_SENDING_BREAK,
	M_DONE,
	M_YOU_ARE_USING,
	M_DONT_KNOW_PROTO,
	M_FILE_TO_SEND,
	M_CHOOSE,
	M_FILE_TO_RECEIVE,
	M_LOG_CLOSED,
	M_LOG_NOT_OPEN,
	M_LOG_OPEN,
	M_END_OF_CONNECT,
	M_SWITCHING_BACK,
	M_POLLING_COLON,
	M_GET_FILE,
	M_ADDRESS_TO_GET_FROM,
	M_PASSWORD_TO_USE,
	M_ARE_YOU_SURE,
	M_YES,
	M_NO,
	M_SEND_FILE,
	M_ADDRESS_TO_SEND_TO,
	M_KILL_MAIL,
	M_ADDRESS_TO_KILL,
	M_KILL_ALL_MAIL,
	M_DRATS,
	M_HE_HUNG_UP,
	M_CORRECTED_ERRORS,
	M_FILE_SENT,
	M_SYNCHRONIZING,
	M_TEMP_NOT_OPEN,
	M_ALREADY_HAVE,
	M_SYNCHRONIZING_EOF,
	M_SYNCHRONIZING_OFFSET,
	M_FILE_RECEIVED,
	M_ORIGINAL_NAME_BAD,
	M_UNEXPECTED_EOF,
	M_REMOTE_SYSTEM,
	M_UNKNOWN_MAILER,
	M_SYSTEM_INITIALIZING,
	M_UNRECOGNIZED_OPTION,
	M_THANKS,
	M_ANOTHER_FINE_PRODUCT,
	M_REMOTE_REFUSED,
	M_ERROR,
	M_CANT,
	M_CPS_MESSAGE,
	M_COMPRESSED_MAIL,
	M_NET_FILE,
	M_ELAPSED,
	M_TROUBLE,
	M_RESENDING_FROM,
	M_SEND_MSG,
	M_UPDATE,
	M_FILE,
	M_REQUEST,
	M_EXECUTING,
	M_CARRIER_REQUEST_ERR,
	M_FREQ_LIMIT,
	M_EVENT_OVERRUN,
	M_NO_AVAIL,
	M_NO_ABOUT,
	M_OKFILE_ERR,
	M_FREQ_PW_ERR,
	M_RECEIVE_MSG,
	M_TIMEOUT,
	M_CHECKSUM,
	M_CRC_MSG,
	M_JUNK_BLOCK,
	M_ON_BLOCK,
	M_FIND_MSG,
	M_READ_MSG,
	M_SEEK_MSG,
	M_SHRT_MSG,
	M_CLOSE_MSG,
	M_UNLINK_MSG,
	M_WRITE_MSG,
	M_SKIP_MSG,
	M_PHONE_HELP,
	M_PHONE_HELP2,
	M_LAST_SET,
	M_CURRENT_PHONES,
	M_INPUT_COMMAND,
	M_ELEMENT_CHOSEN,
	M_CURRENTLY_CONTAINS,
	M_PHONE_HELP3,
	M_PHONE_HELP4,
	M_SET_READ_ERROR,
	M_SET_CLOSE_ERR,
	M_SET_LOADED,
	M_SET_WRITE_ERROR,
	M_SET_SAVED,
	M_PRESS_ENTER,
	M_STARTING_SCAN,
	M_CONNECTED_TO_ITEM,
	M_DEBRIS,
	M_LONG_PACKET,
	M_Z_IGNORING,
	M_OUT_OF_DISK_SPACE,
	M_RECEIVING,
	M_Z_INITRECV,
	M_BAD_POS,
	M_Z_RZ,
	M_J_BAD_PACKET,
	M_SEND,
	M_RECV,
	M_OTHER_DIED,
	M_GOING_ONE_WAY,
	M_REFUSING,
	M_UNKNOWN_PACKET,
	M_SESSION_ABORT,
	M_SENDING,
	M_NO_LENGTH,
	M_FINISHED_PART,
	M_SAVING_PART,
	M_REMOTE_CANT_FREQ,
	M_MAIN_HELP,
	M_MAIN_HELP1,
	M_MAIN_HELP2,
	M_MAIN_HELP3,
	M_MAIN_HELP4,
	M_MAIN_HELP5,
	M_MAIN_HELP6,
	M_MAIN_HELP7,
	M_MAIN_HELP8,
	M_MAIN_HELP9,
	M_MAIN_HELP_LAST,
	M_MACRO_HELP,
	M_PRESS_ANYKEY,
	M_MAILER_HELP,
	M_MAILER_HELP1,
	M_MAILER_HELP2,
	M_MAILER_HELP3,
	M_MAILER_HELP4,
	M_MAILER_HELP5,
	M_MAILER_HELP6,
	M_MAILER_HELP7,
	M_MAILER_HELP8,
	M_MAILER_HELP9,
	M_MAILER_HELP10,
	M_MAILER_HELP11,
	M_MAILER_HELP12,
	M_MAILER_HELP13,
	M_MAILER_HELP14,
	M_MAILER_HELP15,
	M_MAILER_HELP_LAST,
	M_BLANK_HELP,
	M_EDITOR_HELP,
	M_SHELL_HELP,
	M_DRIVER_DEAD_1,
	M_DRIVER_DEAD_2,
	M_DRIVER_DEAD_3,
	M_FAILED_CREATE_FLAG,
	M_CREATED_FLAGFILE,
	M_THIS_ADDRESS_LOCKED,
	M_BAD_CLEAR_FLAGFILE,
	M_CLEARED_FLAGFILE,
	M_FAILED_CLEAR_FLAG,
	M_BYTE_LIMIT,
	M_REFRESH_NODELIST,
	M_NO_SIGINT,
	M_FILTER,
	M_NEXT_EVENT,
	M_EVENT_FLAGS,
/* Extra for ST */
	M_NOTHING_IN_OUTBOUND_STATUS,
	M_DO_SHELL,
	M_ANYKEY,
	M_NO_CHANGE,
	M_DO_CHANGE,
	M_ADDRESS_TO_CHANGE,
	M_CHANGE_FROM,
	M_CHANGE_TO,
	M_KEYWARN1,
	M_KEYWARN2,
	M_KILLING,
	M_BEGIN,
	M_FREEMEM,
	M_END,
	M_OTHERPROC,
	M_EXECSHELL,
	M_RETSHELL,
	M_NOYOOHOO,
/* Extra for EMSI */
	M_EMSI_NOMEM,
	M_EMSI_METHOD,
	M_EMSI_END,
	M_NONE,
	M_TIME_LIMIT,
	M_NODE_COST,
	M_REMOTE_ADDRESS,
	M_EMSI_SYSTEM,
	M_EMSI_LOCATION,
	M_EMSI_SYSOP,
	M_EMSI_PHONE,
	M_EMSI_BAUD,
	M_EMSI_FLAGS,
	M_SENDING_FOR,
	M_EMSI_FILES,
	M_NOPASSWORD,
	X_LAST
} MSGTYPE;

#define X_TOTAL_MSGS (X_LAST-1)

#endif	/* H_MSGS */
/* The end */
