/*
 * include.c
 *
 * Some precompiled header files
 */

#include <stdio.h>
#include <signal.h>
#include <ctype.h>
#include <string.h>
#include <dos.h>	/* */
#include <stdlib.h> /* */
#include <conio.h>
#include <errno.h>
#include <fcntl.h>
#include <portab.h>
#include <time.h>


/* Just the major ones */

#include "com.h"
#include "xfer.h"
#include "zmodem.h"
#include "keybd.h"
#include "sbuf.h"
#include "sched.h"
#include "externs.h"
#include "prototyp.h"

