cp /joopk/sources/bink/node/binkley.cfg /joopk/sources/bink/binkley.cfg
chown joopk /joopk/sources/bink/binkley.cfg
cp /joopk/sources/bink/node/rfmail.cf /usr/local/lib/fnet/rfmail.cf
chown uucp /usr/local/lib/fnet/rfmail.cf
cp /joopk/sources/bink/node/bink /joopk/sources/bink/bink
chown uucp /joopk/sources/bink/bink
chmod u+s /joopk/sources/bink/bink
cp /joopk/sources/bink/node/NODELIST.DAT /usr/local/spool/fnet/nodelist/NODELIST.DAT
chown joopk /usr/local/spool/fnet/nodelist/NODELIST.DAT
cp /joopk/sources/bink/node/NODELIST.IDX /usr/local/spool/fnet/nodelist/NODELIST.IDX
chown joopk /usr/local/spool/fnet/nodelist/NODELIST.IDX