#	Default actions.

RM = Rm

.c.o:
	$(CC) $(CFLAGS) -o$@ $<

.c.prg:
	$(CC) $(CFLAGS) -l -o$@ $<

.c.ttp:
	$(CC) $(CFLAGS) -l -o$@ $<

.c.tos:
	$(CC) $(CFLAGS) -l -o$@ $<

.s.o:
	$(AS) $(ASFLAGS) $<

.DEFAULT:
	@echo Made $@

#	Default macro definitions.

.SUFFIXES: .prg .ttp .tos .o .c .s

CC = lc
AS = asm
ASFLAGS =
CFLAGS =
LN = clink
LINK = $(LN)
LIBS = lc.lib
LNFLAGS = FROM c.o
